/*===============================================================================
INCLUDES AND VARIABLE DEFINITIONS
=============================================================================== */
#include "ExSample.h"		// Our header to distribute for our class

/*===============================================================================
CLASS DEFINITIONS
- If you have more than one class in this module, you should list them here.
=============================================================================== */
typedef struct _ExSample
{
	// Declare our VTable
	// by doing so, we have placed function pointers
	// in the begininng of this structure
	DECLARE_VTBL(IExSample)

	// Class member variables
	uint32			m_nRefs;		// References to us
	IShell *		m_pIShell;		// copy of Shell pointer
	IDisplay *		m_pIDisplay;	// Display interface pointer
	IModule *		m_pIModule;		// IModule interface pointer
}ExSample;

// Exported AEEClsCreateInstance

extern int			AEEClsCreateInstance(AEECLSID, IShell*, IModule*, void**);

/*-------------------------------------------------------------------
Function Prototypes
-------------------------------------------------------------------*/
static uint32		ExSample_AddRef(IExSample * po);
static uint32		ExSample_Release(IExSample * po);
static int			ExSample_PrintString(IExSample * po, char *strValue);

int ExSample_New(int16 nSize, IShell *pIShell, IModule* pIModule, IModule ** ppMod)
{
	ExSample *			   pMe = NULL;
	VTBL(IExSample) *	modFuncs;

	if( !ppMod || !pIShell || !pIModule )
		return EFAILED;

	*ppMod = NULL;

	// Allocate memory for the ExSample object
	if( nSize < sizeof(ExSample) )
		nSize += sizeof(ExSample);

	if( (pMe = (ExSample *)MALLOC(nSize + sizeof(IExSampleVtbl))) == NULL )
		return ENOMEMORY;

	// Allocate the vtbl and initialize it. Note that the modules and apps must not
	// have any static data. Hence, we need to allocate the vtbl as well.
	modFuncs = (IExSampleVtbl *)((byte *)pMe + nSize);
	//Initialize individual entries in the VTBL
	modFuncs->AddRef			= ExSample_AddRef;
	modFuncs->Release			= ExSample_Release;
	modFuncs->PrintString		= ExSample_PrintString;
	
	// initialize the vtable
	INIT_VTBL(pMe, IModule, *modFuncs);

	// initialize the data members
	pMe->m_nRefs      = 1;
	pMe->m_pIShell    = pIShell;
	pMe->m_pIModule   = pIModule;

	// Add References and get IDisplay
	ISHELL_AddRef(pIShell);
	IMODULE_AddRef(pIModule);
	if( ISHELL_CreateInstance(pIShell, AEECLSID_DISPLAY, (void **)&pMe->m_pIDisplay) != SUCCESS )
		return EFAILED;

	// Set the pointer in the parameter
	*ppMod = (IModule*)pMe;

	return AEE_SUCCESS;
}

/*===========================================================================

Function:  ExSample_AddRef()

Description:
		This function increases the reference count for the ExSample object

Prototype:
	uint32 ExSample_AddRef(IExSample * po)

Parameters:
   po: Pointer to IExSample interface whos reference count is to be incremented

Return Value:
	The updated reference count

Comments:  None

Side Effects: None

==============================================================================*/
static uint32 ExSample_AddRef(IExSample * po)
{
   return (++((ExSample *)po)->m_nRefs);
}
/*===========================================================================

Function:  ExSample_Release()

Description:
		This function decreases the reference count for the ExSample object. If
		the reference count reaches zero, it does the required cleanup

Prototype:
	uint32 ExSample_Release(IExSample* po)

Parameters:
   po: Pointer to the IExSample interface whose reference count needs to be
	decremented.

Return Value:
   The updated reference count

Comments:  None

Side Effects: None

==============================================================================*/
static uint32 ExSample_Release(IExSample * po)
{
	ExSample *		pMe = (ExSample *)po;

	if( --pMe->m_nRefs != 0 )
		return pMe->m_nRefs;

	// Ref count is zero. So, release memory associated with this object.

	if( pMe->m_pIDisplay )
		IDISPLAY_Release(pMe->m_pIDisplay);

	// Release interfaces
	ISHELL_Release(pMe->m_pIShell);
	IMODULE_Release(pMe->m_pIModule);
	//Free the object itself
	FREE_VTBL(pMe, IModule);
	FREE( pMe );

	return 0;
}


/*===========================================================================

Function:  AEEClsCreateInstance()

Description:
		This function decreases the reference count for the ExSample object. If
		the reference count reaches zero, it does the required cleanup.
		Calls a helper function to initialize the class members and VTable.

Prototype:
	uint32 AEEClsCreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)

Parameters:
   ClsId: Clsid passed in
   pIShell: IShell pointer
   po: IModule pointer
   ppObj[out]: Pointer to a pointer of the class object to fill

Return Value:
   AEE_SUCCESS on pass
   Error code on failure
Comments:  None

Side Effects: None

==============================================================================*/
int AEEClsCreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)
{
	*ppObj = NULL;
		
	if( ClsId == AEECLSID_EXTENSION_KAMJA )
	{
		if( ExSample_New(sizeof(ExSample), pIShell, po, (IModule **)ppObj) == SUCCESS )
			return AEE_SUCCESS;
	}
	return EFAILED;

}

static int	ExSample_PrintString(IExSample * po, char *strValue)
{
	ExSample *	   pMe = (ExSample *) po;
	AECHAR szBuf[200] = {0};

	if( !pMe || !pMe->m_pIDisplay )
		return EFAILED;
	
	STREXPAND((byte *)strValue, STRLEN(strValue), szBuf, sizeof(szBuf));

	IDISPLAY_DrawText(pMe->m_pIDisplay, AEE_FONT_BOLD, szBuf, -1, 0, 0, 0, IDF_ALIGN_CENTER | IDF_ALIGN_MIDDLE);

	return SUCCESS;

}